/**
  ******************************************************************************
  * @file main.c
  * @brief This file contains the main function for real time keeping example.
  * @author STMicroelectronics - MCD Application Team
  * @version V1.1.1
  * @date 23/11/2009
  ******************************************************************************
  *
  * THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2008 STMicroelectronics</center></h2>
  * @image html logo.bmp
  ******************************************************************************
  */

// Includes ------------------------------------------------------------------
#include "stm8s.h"
#include "stm8s_it.h"
#include "string.h"
#include "rtc.h"

// User define ---------------------------------------------------------------
// Private typedef -----------------------------------------------------------
// Private define ------------------------------------------------------------
// Private macros ------------------------------------------------------------
#define external_service		((GPIOC->IDR & 1) == 0)
#define beeper_at_lowest_speed { BEEP->CSR= 0x1E & BEEP_CSR_BEEPDIV; }

// Private constants ---------------------------------------------------------
const st_time TIME_INIT= { 0,0,0,0 };

// Private variables ---------------------------------------------------------
	st_fx_time fx_time;		// variable for time keeping	
	u32 lsi_fqcy;					// computed LSI frequency
		
// Private macros ------------------------------------------------------------
#define copy_time(a,b)		memcpy((st_time *)(a),(const st_time *)(b), sizeof(st_time))
	
// User variables ------------------------------------------------------------
	st_time real_time;
	u16 beep_fqcy;
	u32 edge_day_overflow_limit;
	
// Private functionos --------------------------------------------------------
#ifndef _COSMIC_
void delay(u16 dly) { 
  while(dly--);
}
#endif
/**
  * @brief Measure the LSI frequency using timer IC1 and update the calibration registers.
  * @par Parameters:
  * None
  * @retval 
	* None
  * @par Required preconditions:
  * It is recommanded to use a timer clock frequency of at least 10MHz in order 
	* to obtain a better in the LSI frequency measurement.
  */
u32 LSIMeasurment(void)
{
  
  u32 lsi_freq_hz = 0x0;
  u32 fmaster = 0x0;
  u16 ICValue1 = 0x0;
  u16 ICValue2 = 0x0;

  // Get master frequency 
  fmaster = CLK_GetClockFreq();

  // Enable the LSI measurement: LSI clock connected to timer Input Capture 1 
  AWU->CSR |= AWU_CSR_MSR;

#if defined (STM8S903) || defined (STM8S103)
  // Measure the LSI frequency with TIMER Input Capture 1 
  
  // Capture only every 8 events!!! 
  // Enable capture of TI1 
	TIM1_ICInit(TIM1_CHANNEL_1, TIM1_ICPOLARITY_RISING, TIM1_ICSELECTION_DIRECTTI, TIM1_ICPSC_DIV8, 0);
	
  // Enable TIM1 
  TIM1_Cmd(ENABLE);
  
  // wait a capture on cc1 
  while((TIM1->SR1 & TIM1_FLAG_CC1) != TIM1_FLAG_CC1);
  // Get CCR1 value
  ICValue1 = TIM1_GetCapture1();
  TIM1_ClearFlag(TIM1_FLAG_CC1);
  
  // wait a capture on cc1 
  while((TIM1->SR1 & TIM1_FLAG_CC1) != TIM1_FLAG_CC1);
  // Get CCR1 value
  ICValue2 = TIM1_GetCapture1();
  TIM1_ClearFlag(TIM1_FLAG_CC1);
  
  // Disable IC1 input capture 
  TIM1->CCER1 &= (u8)(~TIM1_CCER1_CC1E);
  // Disable timer2 
  TIM1_Cmd(DISABLE);
  
#else  
  // Measure the LSI frequency with TIMER Input Capture 1 
  
  // Capture only every 8 events!!! 
  // Enable capture of TI1 
  TIM3_ICInit(TIM3_CHANNEL_1, TIM3_ICPOLARITY_RISING, TIM3_ICSELECTION_DIRECTTI, TIM3_ICPSC_DIV8, 0);

  // Enable TIM3 
  TIM3_Cmd(ENABLE);

	// wait a capture on cc1 
  while ((TIM3->SR1 & TIM3_FLAG_CC1) != TIM3_FLAG_CC1);
	// Get CCR1 value
  ICValue1 = TIM3_GetCapture1();
  TIM3_ClearFlag(TIM3_FLAG_CC1);

  // wait a capture on cc1 
  while ((TIM3->SR1 & TIM3_FLAG_CC1) != TIM3_FLAG_CC1);
    // Get CCR1 value
  ICValue2 = TIM3_GetCapture1();
	TIM3_ClearFlag(TIM3_FLAG_CC1);

  // Disable IC1 input capture 
  TIM3->CCER1 &= (u8)(~TIM3_CCER1_CC1E);
  // Disable timer3 
  TIM3_Cmd(DISABLE);
#endif

  // Compute LSI clock frequency 
  lsi_freq_hz = (8 * fmaster) / (ICValue2 - ICValue1);
  
  // Disable the LSI measurement: LSI clock disconnected from timer Input Capture 1 
  AWU->CSR &= (u8)(~AWU_CSR_MSR);

 return (lsi_freq_hz);
}
//* ---------------------------------------------------------------------------*/
/**
  * @brief Software self calibration of beeper frequency
  * @par Parameters: none
  * @retval none
  */
u16 measure_beep_fqcy(void) {
#ifdef MEASURE_ON_HSE										// optional switch to HSE
	CLK_SYSCLKConfig(CLK_PRESCALER_CPUDIV1);			// set fcpu= fmaster/2
	CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO,CLK_SOURCE_HSE,DISABLE,CLK_CURRENTCLOCKSTATE_ENABLE);
#else
	CLK_SYSCLKConfig(CLK_PRESCALER_CPUDIV1);			// set fcpu= fmaster
	CLK_HSIPrescalerConfig(CLK_PRESCALER_HSIDIV1);	// set fhsidiv (fmaster) at maximum speed
#endif /* MEASURE_ON_HSE */

#ifndef _COSMIC_
  delay(1000);            // !!! delay should be inserted due to the bug in firmware library V1.1.1 !!!
#endif                    // (test of the end of clock switching fails when Raisonance compiler is used)

	lsi_fqcy= LSIMeasurment();              // LSI measurement
	
#ifdef MEASURE_ON_HSE										// optional switch back to HSI
	CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO,CLK_SOURCE_HSI,DISABLE,CLK_CURRENTCLOCKSTATE_ENABLE);
#else
	CLK_SYSCLKConfig(CLK_PRESCALER_CPUDIV1);			// set fcpu= fmaster
#endif /* MEASURE_ON_HSE */

	return (lsi_fqcy/8 + 16)/32;								// result of division by 32 is rounded (+16)
}
//* ---------------------------------------------------------------------------*/
/**
  * @brief Incremental real time conversion routine, it adds one second to time structure registers
  * @par Parameters:
  *  *t: pointer to time structure
  * @retval none
  */
#ifdef KEEP_RT
void rt_one_second_increment (st_time *t) {
	if(++t->second > 59) {
		t->second= 0;
		if(++t->minute > 59) {
			t->minute= 0;
			if(++t->hour > 23) {
				t->hour= 0;
			}
		}
	}
}
#endif /* KEEP_RT */
//* ---------------------------------------------------------------------------*/
/**
  * @brief Update real time conversion routine, it updates of time structure registers
  * @par Parameters:
  *  *t: pointer to user time structure
  * @retval none
  */
void rt_update (st_time *t) {
	u32 tmp_mod;
	
	tmp_mod= (fx_time.edge_day_counter *  64) / (lsi_fqcy / 4);	// update time registers		
	t->hour= tmp_mod / SECONDS_IN_A_HOUR;
	tmp_mod%= SECONDS_IN_A_HOUR;
	t->minute= tmp_mod / SECONDS_IN_A_MINUTE;
	t->second= tmp_mod % SECONDS_IN_A_MINUTE;
	
	while(fx_time.day_overflow_counter != 0) {
		--fx_time.day_overflow_counter;
		if(++t->day > 6) t->day= 0;
	}
#ifdef KEEP_RT	
	fx_time.second_overflow_counter= 0;				// clear all second events
#endif /* KEEP_RT */
}
//* Public functions ----------------------------------------------------------*/
/**
  * @brief Main function - initialization and main loop with check of all time intervals passed
  * @par Parameters: none
  * @retval none
  */  
void main(void) {
// 								*** BEEP FREQUENCY CALIBRATION & SETTING ***
	beep_fqcy= measure_beep_fqcy();	// LSI measurement
	beeper_at_lowest_speed;				// setting the lowest possible frequency output for beeper
		
// 								*** BEEP & LSI OUTPUT ENABLE ***
	BEEP_Cmd(ENABLE);						// Beeper signal output enable at BEEP pin (PD4)
	CLK_CCOConfig(CLK_OUTPUT_LSI);	// LSI output enable on CLK_CCO pin (PE0)
	
// 								*** GPIO & EXTI INIT ***
	GPIO_Init(GPIOD, GPIO_PIN_4, GPIO_MODE_IN_FL_IT);	// PD4 external interrupt pin - float input
	EXTI_SetExtIntSensitivity(EXTI_PORT_GPIOD, EXTI_SENSITIVITY_FALL_ONLY);
	
// 								*** MAIN LOOP INITIAL ***
	copy_time(&real_time, &TIME_INIT);			// real time structure initialization
	
	edge_day_overflow_limit= (lsi_fqcy * 675 + 1) / 2;	// initialization of a day overflow limit
	
	enableInterrupts();
	
// 								*** MAIN LOOP ***
	while(1) {
#ifdef KEEP_RT
    while(fx_time.day_overflow_counter != 0) {
      --fx_time.day_overflow_counter;             // one day event service
      if(++real_time.day > 6) real_time.day= 0;   // increment of the day counter
      real_time.second= real_time.minute= real_time.hour= 0;  // init real time structure at begin of a day
    }
		while(fx_time.second_overflow_counter != 0) {
			--fx_time.second_overflow_counter;	        // one second event service
			rt_one_second_increment(&real_time);        // keeping actual time information
		}
#endif /* KEEP_RT */
		if(external_service)
			rt_update(&real_time);					// external event service - update time information
#ifndef IN_DEBUG
		halt();											      // wait for next event in halt mode
		nop();
#endif
	}
}
/**
  * @brief Reports the name of the source file and the source line number where
  * the assert error has occurred.
  * User can add his own implementation to report the file name and line number.
  * ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line)
  * @retval void None
  * @par Required preconditions:
  * None
  * @par Called functions:
  * None
  */
#ifdef FULL_ASSERT
void assert_failed(u8 *file, u16 line)
#else
void assert_failed(void)
#endif
{
  // Put enter your own code to manage an assert error 
}

